//+------------------------------------------------------------------+
//|                                                    FibLevels.mq4 |
//|                                                          mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property version   "1.13"
#property strict
#property indicator_chart_window
//+------------------------------------------------------------------+
//| License Stuff                                                    |
//+------------------------------------------------------------------+
int      accountNumber = 0;               // 0 = Allow all accounts
datetime expirationDate = D'2019.12.31';  // Expiration date (yyyy.mm.dd)
//+------------------------------------------------------------------+
//| DO NOT CHANGE ANYTHING BELOW HERE - UNLESS YOU KNOW WHAT YOU DO! |
//+------------------------------------------------------------------+
#include <mwfx108/BaseLevels.mqh>
#include <mwfx108/ChartTools.mqh>
#include <mwfx108/FibLevels.mqh>
//+------------------------------------------------------------------+
//| Variables                                                        |
//+------------------------------------------------------------------+
FIBLEVEL _levels[];
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
//---

   RemoveChartObjects("FIBLVLS_");
   ArrayFree(_levels);
   
//---
}
//+------------------------------------------------------------------+
//| Custom indicator chart event                                     |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long& lparam,
                  const double& dparam,
                  const string& sparam)
{
   if (id == CHARTEVENT_CLICK || id == CHARTEVENT_CHART_CHANGE)
      Process(TRUE);
}                  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
//---
   
   Process();
   
//--- return value of prev_calculated for next call
   return(rates_total);
}
//+------------------------------------------------------------------+
//| Do all the things.                                               |
//+------------------------------------------------------------------+
void Process(bool force = FALSE)
{
//---
   
   if (Period() >= PERIOD_D1)
      return;
      
   if (CheckLicense())
   {
      static datetime lastCalculation = 0;
      datetime currentTime = iTime(_Symbol, PERIOD_CURRENT, 0);
      if (currentTime > lastCalculation || force)
      {
         lastCalculation = currentTime;
         RemoveChartObjects("FIBLVLS_");
         
         ArrayFree(_levels);
         if (GetFiboLevels(_Symbol, 10, _levels))
            DrawFiboLevels(_levels);
         else
            Comment(StringFormat("Pair \"%s\" is not supported!", _Symbol));
      }
      
      if (ArraySize(_levels) > 0)
         DrawSessionInformation(_levels);
   }
   else
      RemoveChartObjects("FIBLVLS_");
      
//---
}
//+------------------------------------------------------------------+
//| Check account number and expiration date.                        |
//+------------------------------------------------------------------+
bool CheckLicense()
{
//---

   Comment("");
   
   if (accountNumber != 0 && AccountNumber() != accountNumber)
   {
      Comment("Wrong account number. Please visit www.fxcalcno1.com for more information!");
      return (false);
   }
   
   if (TimeCurrent() >= expirationDate) 
   {
      Comment("This indicator has expired. Please visit www.fxcalcno1.com for more information!");
      return(false);
   }
   
   return (true);	
   
//---
}
//+------------------------------------------------------------------+
