//+------------------------------------------------------------------+
//|                                                     FibLines.mq4 |
//|                                                          mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property version   "1.00"
#property strict
#property indicator_chart_window
//+------------------------------------------------------------------+
//| Variables and Consts                                             |
//+------------------------------------------------------------------+
enum ENUM_INSTRUMENTS {
   Invalid =  0,
   NZDUSD  =  1,
   USDCHF  =  2,
   USDCAD  =  3,
   EURUSD  =  4,
   GBPUSD  =  5,
   AUDUSD  =  6,
   EURAUD  =  7,
   USDJPY  =  8,
   GBPJPY  =  9,
   EURJPY  = 10,
   AUDJPY  = 11
};

const double Baselines[] = {
   0.0127,  // Modifier
   0.6898,  // NZDUSD
   1.2000,  // USDCHF
   1.2000,  // USDCAD
   1.2000,  // EURUSD
   2.0000,  // GBPUSD
   0.8248,  // AUDUSD
   1.4160,  // EURAUD
   106.34,  // USDJPY
   144.12,  // GBPJPY
   125.08,  // EURJPY
    77.15   // AUDJPY
};

double _baseLine = 0.0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   
   ENUM_INSTRUMENTS instrument = GetInstrument(_Symbol);
   if (instrument == Invalid)
   {
      PrintFormat("Symbol not supported: %s");
      return (INIT_FAILED);
   }
   _baseLine = Baselines[instrument];
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator chart event                                     |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long& lparam,
                  const double& dparam,
                  const string& sparam)
{
//---
   if (id == CHARTEVENT_CLICK || id == CHARTEVENT_CHART_CHANGE)
      Process();
//---
}                  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
//---
   Process();
//--- return value of prev_calculated for next call
   return(rates_total);
}
//+------------------------------------------------------------------+
//| Calculate and draw.                                              |
//+------------------------------------------------------------------+
void Process()
{
//---

   static datetime lastRun = 0;
   datetime currentTime = iTime(_Symbol, PERIOD_CURRENT, 0);
   
   if (currentTime > lastRun)
   {
      lastRun = currentTime;
      RemoveObjects("FibLines_");
      
      double levels[];
      GetFiboLevels(_Symbol, 10, _baseLine, levels);
      for (int i = 0; i < ArraySize(levels); i++)
      {
         string objName = "FibLines_" + IntegerToString(i);
         ObjectCreate(objName, OBJ_HLINE, 0, 0, levels[i]);
         ObjectSetInteger(0, objName, OBJPROP_BACK, TRUE);
      }
      
      ArrayFree(levels);
   }

//---
}
//+------------------------------------------------------------------+
//| Translate Symbol to Instrument                                   |
//+------------------------------------------------------------------+
ENUM_INSTRUMENTS GetInstrument(string symbol)
{
//---

   ENUM_INSTRUMENTS result = Invalid;

   string sSymbol = symbol;
   StringToLower(sSymbol);
   for (int i = NZDUSD; i <= AUDJPY; i++)
   {
      string enumInstrument = EnumToString(ENUM_INSTRUMENTS(i));
      StringToLower(enumInstrument);
      if (StringFind(sSymbol, enumInstrument) >= 0 || StringFind(enumInstrument, sSymbol) >= 0)
      {
         result = ENUM_INSTRUMENTS(i);
         break;
      }
   }
   // ----

   return (result);
   
//---
}
//+------------------------------------------------------------------+
//| Get Fibonacci levels of given base line.                         |
//+------------------------------------------------------------------+
void GetFiboLevels(string sSymbol, int numLevels, double basePrice, double &levels[])
{
//---

   double price = Close[0], 
      offset = 0,
      _point = _Point,
      fibPips = 61.0 * _point; //63.5 * _point;
   int _digits = _Digits;
   
   ArrayResize(levels, numLevels * 2);

	if (_digits == 3 || _digits == 5)
		fibPips *= 10;

   if (price < basePrice)
      do {
         offset -= fibPips;
      } while(basePrice + offset > price);
   else
      do {
         offset += fibPips;
      } while(basePrice + offset < price);

   for (int i = -numLevels; i < numLevels; i++)
   {
      double level = NormalizeDouble((basePrice + offset) + (fibPips * i), _digits);
      levels[i + numLevels] = level;
   }
   
//---   
}
//+------------------------------------------------------------------+
//| Remove objects                                                   |
//+------------------------------------------------------------------+
void RemoveObjects(string prefix)
{
   for (int i = ObjectsTotal(); i >= 0; i--)
   {
      string objName = ObjectName(i);
      if (StringFind(objName, prefix) >= 0)
         ObjectDelete(objName);
   }
}
//+------------------------------------------------------------------+
