#define FXCALC_NO_OUTPUT
#define FIBLVLS_NO_OUTPUT
//+------------------------------------------------------------------+
//|                                                        FxFib.mq4 |
//|                                                          mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property version   "1.00"
#property strict
#property indicator_chart_window
//+------------------------------------------------------------------+
//| License Stuff                                                    |
//+------------------------------------------------------------------+
int      _acctNumber = -1;          // -1 = Allow any account
datetime _expDate = D'2019.12.31';  // Expiration date (yyyy.mm.dd)
//+------------------------------------------------------------------+
//| DO NOT CHANGE ANYTHING BELOW HERE - UNLESS YOU KNOW WHAT YOU DO! |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Includes                                                         |
//+------------------------------------------------------------------+
#include <mwfx108/BaseLevels.mqh>
#include <mwfx108/ChartTools.mqh>
#include <mwfx108/FxCalcNo1.mqh>
#include <mwfx108/FibLevels.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
extern string           sep1 = "==== History ====";      // History
extern int              _HistorySessions = 13;           // Show History (Past sessions)
extern string           sep2 = "==== Confluence ====";   // Confluence
extern int              _AreaOfConfluence = 14;          // Area of Confluence (Pips)
extern bool             _CombineLevels = true;           // Combine levels
extern int              _CombineRange = 5;               // Combination range (Pips)
extern string           sep3 = "==== Visual ====";       // Visual
extern color            _LevelColor = C'64,64,64';       // Levels color
extern string           sep4 = "==== Sessions ====";     // Sessions
extern ENUM_LINE_STYLE  _SessLineStyle = STYLE_DOT;      // Style
extern int              _SessLineWidth = 1;              // Width
extern color            _SessLineColor = clrWhite;       // Color
//+------------------------------------------------------------------+
//| Variables                                                        |
//+------------------------------------------------------------------+
struct COMBINED_LEVEL {
   double price;
   int strength;
};
ENUM_INSTRUMENTS _instrument = Invalid;
double _nyClose;
ENUM_ORDER_TYPE _bias;
BaseLevel _fxLevels[];
FIBLEVEL _fibLevels[];
COMBINED_LEVEL _combinedLevels[];
int _lastAcctNumber = 0;
int _acctValid = 0;

int confluenceAreaPoints = 0;
int combinationAreaPoints = 0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
//--- indicator buffers mapping

   MathSrand(GetTickCount());
   _acctValid = MathRand();
   
   _HistorySessions = MathMax(1, _HistorySessions);
   confluenceAreaPoints = _AreaOfConfluence * 10;
   combinationAreaPoints = _CombineRange * 10;
   
//---
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
//---

   Comment("");
   
   ArrayFree(_fxLevels);
   ArrayFree(_fibLevels);
   ArrayFree(_combinedLevels);

   RemoveChartObjects("FXFIB_");

//---
}
//+------------------------------------------------------------------+
//| Custom indicator chart event                                     |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long& lparam,
                  const double& dparam,
                  const string& sparam)
{
//---

   if (id == CHARTEVENT_CLICK || id == CHARTEVENT_CHART_CHANGE)
   {
      double currentPrice = iClose(_Symbol, PERIOD_D1, 0);
      /*DrawLevels(_nyClose, currentPrice, _fxLevels, _bias,
         Display_Mode, Show_Break_Lines);*/
   }

//---
}                  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
//---

   Process();
   
//--- return value of prev_calculated for next call
   return(rates_total);
}
//+------------------------------------------------------------------+
//| Check account number and expiration date.                        |
//+------------------------------------------------------------------+
void CheckLicense()
{
//---

   if (TimeCurrent() >= _expDate)
   {
      _acctValid = MathRand();
      
      string message = "This indicator has expired. Please visit www.fxcalcno1.com to download an update or for more information!";
      Print(message);
      Comment(message);
      
      return;
   }
   
   static bool stateChanged = TRUE;
   int currentAcctNumber = AccountNumber();
   if (!IsConnected() || currentAcctNumber == 0)
   {
      Comment("");
      stateChanged = TRUE;
      _acctValid = MathRand();
      return;
   }
   
   if (currentAcctNumber != _lastAcctNumber)
   {
      _lastAcctNumber = currentAcctNumber;
      _acctValid = MathRand();
      stateChanged = TRUE;
   }

   if (_acctValid < 32768)
   {
      if (currentAcctNumber == _acctNumber || _acctNumber == -1)
      {
         Comment("");
         _acctValid = MathRand() + 32768;
         stateChanged = FALSE;
      }
      else
      {
         if (stateChanged)
         {
            stateChanged = FALSE;
            RemoveChartObjects("FXFIB_");
            
            string message = "Wrong account number. Please visit www.fxcalcno1.com for more information!";
            Print(message);
            Comment(message);
         }
         _acctValid = MathRand();
         return;
      }
   }
   
//---
}
//+------------------------------------------------------------------+
//| Perform calculation and drawing                                  |
//+------------------------------------------------------------------+
void Process()
{
//---

   if (Period() >= PERIOD_D1)
      return;

   CheckLicense();
   if (_acctValid < 32768)
   {
      RemoveChartObjects("FXFIB_");
      return;
   }

   _instrument = GetInstrument(_Symbol);
   if (_instrument == Invalid)
   {
      Comment(StringFormat("Pair \"%s\" is not supported!", _Symbol));
      return;
   }

   static datetime lastCheckTime = 0;
   datetime openTime = iTime(_Symbol, PERIOD_D1, 0);
   double currentPrice = iClose(_Symbol, PERIOD_D1, 0);
   
   double nmh = 0.0;
   for (int i = 0; i < ArraySize(_fxLevels); i++)
   {
      if (_fxLevels[i].type == LEVEL_NMH)
      {
         nmh = _fxLevels[i].price;
         break;
      }
   }
   
   if (openTime > lastCheckTime || currentPrice > nmh)
   {
      lastCheckTime = openTime;

      for (int i = _HistorySessions; i >= 1; i--)
      {      
         _nyClose = iClose(_Symbol, PERIOD_D1, i);
   
         ArrayFree(_fibLevels);
         GetFiboLevels(_Symbol, 10, _fibLevels);
   
         ArrayFree(_fxLevels);
         GetLevels(_Symbol, _nyClose, _bias, _fxLevels);
         
         ArrayFree(_combinedLevels);
         CombineLevels();
         
         DrawLevels(i - 1);
      }
      
      DrawLevels(0);
   }
   
//---
}
//+------------------------------------------------------------------+
//| Sort levels                                                      |
//+------------------------------------------------------------------+
void SortLevels(COMBINED_LEVEL &levels[], int direction = MODE_DESCEND)
{
//---

   COMBINED_LEVEL temp;
   int n, count=0;
   bool swapped = true;
   
   n = ArraySize(levels);
   
   if (direction == MODE_ASCEND)
   {
      while (swapped)
      {
         swapped = false;
         count++;
         for (int i = 1; i < n; i++)
         {
            if (levels[i - 1].price > levels[i].price)
            {
               temp = levels[i];
               levels[i] = levels[i - 1];
               levels[i - 1] = temp;
               swapped = true;
            }
         }
      }
   }
   else if (direction == MODE_DESCEND)
   {
      while (swapped)
      {
         swapped = false;
         count++;
         for (int i = 0; i < n - 1; i++)
         {
            if(levels[i].price < levels[i + 1].price)
            {
               temp = levels[i + 1];
               levels[i + 1] = levels[i];
               levels[i] = temp;
               swapped = true;
            }
         }
      }
   }
   
//---
}
//+------------------------------------------------------------------+
//| Check and combine Fib levels and FxCalc levels                   |
//+------------------------------------------------------------------+
void CombineLevels()
{
//---

   int sz = ArraySize(_fibLevels) + ArraySize(_fxLevels),
      c = 0;
   ArrayResize(_combinedLevels, sz);
   
   double tolerance = confluenceAreaPoints * _Point;

   for (int i = 0; i < ArraySize(_fxLevels); i++)
   {
      double price = _fxLevels[i].price;
      int inRange = 0;
      for (int j = 0; j < ArraySize(_fibLevels); j++)
      {
         if (_fxLevels[i].price + tolerance >= _fibLevels[j]._220 && _fxLevels[i].price - tolerance <= _fibLevels[j]._220)
         {
            price += _fibLevels[j]._220;
            inRange++;
         }
         if (_fxLevels[i].price + tolerance >= _fibLevels[j]._724 && _fxLevels[i].price - tolerance <= _fibLevels[j]._724)
         {
            price += _fibLevels[j]._724;
            inRange++;
         }
         if (_fxLevels[i].price + tolerance >= _fibLevels[j].base && _fxLevels[i].price - tolerance <= _fibLevels[j].base)
         {
            price += _fibLevels[j].base;
            inRange++;
         }
      }
      
      if (inRange > 0)
      {
         price /= inRange + 1;
         _combinedLevels[c].price = price;
         _combinedLevels[c].strength = inRange;
         c++;
      }
      else
      {
         _combinedLevels[c].price = _fxLevels[i].price;
         _combinedLevels[c].strength = 1;
         c++;
      }
   }
   
   ArrayResize(_combinedLevels, c);
   
   if (_CombineLevels) 
   {
      for (int pass = 0; pass < 3; pass++)
      {
         COMBINED_LEVEL temp[];
         ArrayResize(temp, c);
         c = 0;
         tolerance = combinationAreaPoints * _Point;
         for (int i = 0; i < ArraySize(_combinedLevels); i++)
         {
            double price = _combinedLevels[i].price;
            int inRange = 0;
            for (int j = i + 1; j < ArraySize(_combinedLevels); j++)
            {
               if (_combinedLevels[i].price + tolerance >= _combinedLevels[j].price && _combinedLevels[i].price - tolerance <= _combinedLevels[j].price)
               {
                  price += _combinedLevels[j].price;
                  inRange++;
               }
            }
            
            if (inRange > 0)
            {
               price /= inRange + 1;
               temp[c].price = price;
               temp[c].strength = _combinedLevels[i].strength + inRange;
               c++;
            }
            else
            {
               temp[c].price = _combinedLevels[i].price;
               temp[c].strength = _combinedLevels[i].strength;
               c++;
            }
         }
      
         ArrayResize(temp, c);
         SortLevels(temp);
         ArrayFree(_combinedLevels);
         ArrayCopy(_combinedLevels, temp);
         ArrayFree(temp);
      }
   }

//---
}
//+------------------------------------------------------------------+
//| Draw combined levels                                             |
//+------------------------------------------------------------------+
void DrawLevels(int period)
{
//---

   RemoveChartObjects("FXFIB_" + IntegerToString(period));

   datetime sessStart = iTime(_Symbol, PERIOD_D1, period),
      sessEnd = sessStart + PeriodSeconds(PERIOD_D1);
   color clr = _LevelColor;
   
   CreateVLine("FXFIB_" + IntegerToString(period) + "_SessionStart", sessStart, _SessLineStyle, _SessLineColor, _SessLineWidth);
   CreateVLine("FXFIB_" + IntegerToString(period) + "_SessionEnd", sessEnd, _SessLineStyle, _SessLineColor, _SessLineWidth);

   for (int i = 0; i < ArraySize(_combinedLevels); i++)
   {
      string objName = "FXFIB_" + IntegerToString(period) + "_" + IntegerToString(i);
      double price = _combinedLevels[i].price;
      CreateTLine(objName, sessStart, price, sessEnd, price, FALSE, STYLE_SOLID, clr, _combinedLevels[i].strength);
   }

//---
}
//+------------------------------------------------------------------+
