//+------------------------------------------------------------------+
//|                                                   BaseLevels.mqh |
//|                                              	         mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property strict
//+------------------------------------------------------------------+
//| Constants                                                        |
//+------------------------------------------------------------------+
enum ENUM_INSTRUMENTS {
   Invalid =  0,
   NZDUSD  =  1,
   USDCHF  =  2,
   USDCAD  =  3,
   EURUSD  =  4,
   GBPUSD  =  5,
   AUDUSD  =  6,
   EURAUD  =  7,
   USDJPY  =  8,
   GBPJPY  =  9,
   EURJPY  = 10,
   AUDJPY  = 11,
   GOLD    = 12,
   EURCAD  = 13,
   DXY     = 14
};
enum ENUM_ENTRIES_SELL {
   NPU = 0,
   PCU = 1,
   BCU = 2,
   BHR = 3,
   VIP = 4,
    R1 = 5
};
enum ENUM_ENTRIES_BUY {
   NPD = 0,
   PCD = 1,
   BCD = 2,
   BLR = 3,
    S1 = 4,
   KPT = 5
};
enum ENUM_HBO_LEVELS {
   HBO3 = 0,
   HBO2 = 1,
   HBO1 = 2,
};
enum ENUM_LBO_LEVELS {
   LBO3 = 0,
   LBO2 = 1,
   LBO1 = 2,
};
enum ENUM_BASELEVEL_TYPE {
   LEVEL_NYC         = 0,
   LEVEL_PIVOT       = 1,
   LEVEL_SENTIMENT   = 2,
   LEVEL_SELL        = 3,
   LEVEL_STP         = 4,
   LEVEL_BUY         = 5,
   LEVEL_BTP         = 6,
   LEVEL_HBO         = 7,
   LEVEL_LBO         = 8,
   LEVEL_NMH         = 9
};
const string LEVEL_LABELS[] = {
   "NYC", "Pivot", "Sentiment",
   "NPU", "PCU", "BCU", "BHR", "VIP", "R1",
   "STP1", "STP2", "STP3", "STP4",
   "NPD", "PCD", "BCD", "BLR", "S1", "KPT",
   "BTP1", "BTP2", "BTP3", "BTP4",
   "HBO1", "HBO2", "HBO3",
   "LBO1", "LBO2", "LBO3",
   "NMH"
};
const int INSTRUMENTS_MAX = 15;
const string INSTRUMENTS_MAPPING[] = {
   "",                  // Invalid
   "NZDUSD",            // NZDUSD
   "USDCHF",            // USDCHF
   "USDCAD",            // USDCAD
   "EURUSD",            // EURUSD
   "GBPUSD",            // GBPUSD
   "AUDUSD",            // AUDUSD
   "EURAUD",            // EURAUD
   "USDJPY",            // USDJPY
   "GBPJPY",            // GBPJPY
   "EURJPY",            // EURJPY
   "AUDJPY",            // AUDJPY
   "GOLD,XAUUSD",       // GOLD
   "EURCAD",            // EURCAD
   "#US$indx,DXY"       // DXY
};
struct BaseLevel {
   double price;
   ENUM_BASELEVEL_TYPE type;
   int index;
   int subIndex;
   //--- Visual
   color lineColor;
   ENUM_LINE_STYLE lineStyle;
   int lineWidth;
};
const double BaseLevels[] = {
   0.0127,  // Modifier
   0.6898,  // NZDUSD
   1.2000,  // USDCHF
   1.2000,  // USDCAD
   1.2000,  // EURUSD
   2.0000,  // GBPUSD
   0.8248,  // AUDUSD
   1.4160,  // EURAUD
   106.34,  // USDJPY
   144.12,  // GBPJPY
   125.08,  // EURJPY
    77.15,  // AUDJPY
  1407.79,  // GOLD
   1.4476,  // EURCAD
   69.479   // DXY
};
const double Multipliers[] = {
     0,  // Modifier
     1,  // NZDUSD
     1,  // USDCHF
     1,  // USDCAD
     1,  // EURUSD
     1,  // GBPUSD
     1,  // AUDUSD
     1,  // EURAUD
   100,  // USDJPY
   100,  // GBPJPY
   100,  // EURJPY
   100,  // AUDJPY
  1000,  // GOLD
     1,  // EURCAD
   100   // DXY
};
const double TP_Steps_Buy[]  = {
   0.0042,
   0.0059,
   0.0097,
   0.0127
};
const double TP_Steps_Sell[] = {
   0.0042,
   0.0068,
   0.0097,
   0.0127
};
const double Levels_Sell[] = {
   0.0165,     // NPU
   0.0127,     // PCU
   0.0097,     // BCU
   0.0055,     // BHR
   0.0035,     // VIP
   0.0023      // R1
};
const double Levels_Buy[] = {
   0.0165,     // NPD
   0.0127,     // PCD
   0.0097,     // BCD
   0.0055,     // BLR
   0.0023,     // S1
   0.0014      // KPT
};
const double PV[] = {
   0.6667,
   0.3333
};
const double MS[] = {
   0.25,
   0.75
};
const double HB1[] = {
   0.3333,
   0.6667
};
const double LB1[] = {
   0.25,
   0.75
};
//+------------------------------------------------------------------+
//| Translate Symbol to Instrument                                   |
//+------------------------------------------------------------------+
ENUM_INSTRUMENTS GetInstrument(string symbol)
{
//---

   ENUM_INSTRUMENTS result = Invalid;

   ushort sepCode = StringGetCharacter(",", 0);
   string sSymbol = symbol;
   StringToLower(sSymbol);
   for (int i = 1; i < INSTRUMENTS_MAX; i++)
   {
      string symbolMapping = INSTRUMENTS_MAPPING[i];
      StringToLower(symbolMapping);

      string pairs[];
      StringSplit(symbolMapping, sepCode, pairs);
      int size = ArraySize(pairs);
      for (int j = 0; j < size; j++)
      {
         if (StringFind(pairs[j], sSymbol) >= 0 || StringFind(sSymbol, pairs[j]) >= 0)
         {
            result = ENUM_INSTRUMENTS(i);
            break;
         }
      }
      
      if (result != Invalid)
         break;
   }

   return (result);
   
//---
}
//+------------------------------------------------------------------+
