//+------------------------------------------------------------------+
//|                                                   ChartTools.mqh |
//|                                                          mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property strict
//+------------------------------------------------------------------+
//| Const                                                            |
//+------------------------------------------------------------------+
enum ENUM_ANCHOR_WRAPPER {
   Right = ANCHOR_RIGHT,
   Left = ANCHOR_LEFT
};
//+------------------------------------------------------------------+
//| Create a horizontal line with label.                             |
//+------------------------------------------------------------------+
void CreateHLine(string objName, string objLabel, double price,
   ENUM_ANCHOR_WRAPPER labelAnchor = Left, ENUM_LINE_STYLE style = STYLE_SOLID, 
   color clr = clrRed, int width = 1)
{
//---

   if (ArraySize(Time) < 0)
      return;
   
   string objLabelName = objName + "_Label";
   
   if (ObjectFind(objName) < 0)
   {
      ObjectCreate(objName, OBJ_HLINE, 0, 0, price);
      ObjectSetInteger(0, objName, OBJPROP_BACK, true);
      ObjectSetInteger(0, objName, OBJPROP_COLOR, clr);
      ObjectSetInteger(0, objName, OBJPROP_WIDTH, width);
      ObjectSetInteger(0, objName, OBJPROP_STYLE, style);
      ObjectSetInteger(0, objName, OBJPROP_SELECTABLE, false);
      ObjectSetInteger(0, objName, OBJPROP_SELECTED, false);
      ObjectSetInteger(0, objName, OBJPROP_HIDDEN, true);
      ObjectSetDouble(0, objName, OBJPROP_PRICE1, price);
   }
   else
   {
      ObjectSetDouble(0, objName, OBJPROP_PRICE1, price);
   }

   ENUM_ANCHOR_POINT anchor;
   ENUM_BASE_CORNER corner;
   if (labelAnchor == Right)
   {
      anchor = ANCHOR_RIGHT;
      corner = CORNER_RIGHT_UPPER;
   }
   else
   {
      anchor = ANCHOR_LEFT;
      corner = CORNER_LEFT_UPPER;
   }

   if (StringLen(objLabel) > 0)
      CreateLabel(objLabel, objLabelName, clr, price, anchor, corner);
   
//---
}
//+------------------------------------------------------------------+
//| Create a label.                                                  |
//+------------------------------------------------------------------+
void CreateLabel(string label, string name, color clr, double price, ENUM_ANCHOR_POINT anchor, ENUM_BASE_CORNER corner)
{
//---

   if (price != EMPTY_VALUE && ArraySize(Time) < 0)
      return;

   string objLabel = label,
      objLabelName = name;

   int x, y, w, h;
   TextGetSize(objLabel, w, h);
   
   if (price == EMPTY_VALUE)
   {
      x = 0;
      y = h;
   }
   else
   {
      ChartTimePriceToXY(0, 0, Time[0], price, x, y);
      x = 0;
   }

   if (ObjectFind(objLabelName) < 0)
   {   
      ObjectCreate(objLabelName, OBJ_LABEL, 0, 0, 0);
      ObjectSetInteger(0, objLabelName, OBJPROP_BACK, true);
      ObjectSetInteger(0, objLabelName, OBJPROP_COLOR, clr);
      ObjectSetInteger(0, objLabelName, OBJPROP_XDISTANCE, x + 4);
      ObjectSetInteger(0, objLabelName, OBJPROP_YDISTANCE, y - h / 2);
      ObjectSetInteger(0, objLabelName, OBJPROP_XSIZE, w);
      ObjectSetInteger(0, objLabelName, OBJPROP_YSIZE, h);
      ObjectSetInteger(0, objLabelName, OBJPROP_ANCHOR, anchor);
      ObjectSetInteger(0, objLabelName, OBJPROP_CORNER, corner);
      ObjectSetInteger(0, objLabelName, OBJPROP_BACK, false);
      ObjectSetInteger(0, objLabelName, OBJPROP_SELECTABLE, false);
      ObjectSetInteger(0, objLabelName, OBJPROP_SELECTED, false);
      ObjectSetInteger(0, objLabelName, OBJPROP_HIDDEN, true);
      ObjectSetText(objLabelName, objLabel, 8);
   }
   else
   {
      ObjectSetInteger(0, objLabelName, OBJPROP_XDISTANCE, x + 4);
      ObjectSetInteger(0, objLabelName, OBJPROP_YDISTANCE, y - h / 2);
      ObjectSetText(objLabelName, objLabel, 8);
   }

//---
}
//+------------------------------------------------------------------+
//| Create a vertical line.                                          |
//+------------------------------------------------------------------+
void CreateVLine(string objName, datetime time,
   ENUM_LINE_STYLE style = STYLE_SOLID, color clr = clrRed, int width = 1)
{
//---
   
   if (ObjectFind(objName) < 0)
   {
      ObjectCreate(objName, OBJ_VLINE, 0, time, 0);
      ObjectSetInteger(0, objName, OBJPROP_BACK, true);
      ObjectSetInteger(0, objName, OBJPROP_COLOR, clr);
      ObjectSetInteger(0, objName, OBJPROP_WIDTH, width);
      ObjectSetInteger(0, objName, OBJPROP_STYLE, style);
      ObjectSetInteger(0, objName, OBJPROP_SELECTABLE, false);
      ObjectSetInteger(0, objName, OBJPROP_SELECTED, false);
      ObjectSetInteger(0, objName, OBJPROP_HIDDEN, true);
   }
   else
   {
      ObjectSetInteger(0, objName, OBJPROP_TIME1, time);
   }
   
//---
}
//+------------------------------------------------------------------+
//| Create a trend line.                                             |
//+------------------------------------------------------------------+
void CreateTLine(string objName, datetime time1, double price1, datetime time2, double price2, bool ray = FALSE,
   ENUM_LINE_STYLE style = STYLE_SOLID, color clr = clrRed, int width = 1, int window = 0)
{
//---
   
   if (ObjectFind(objName) < 0)
   {
      ObjectCreate(objName, OBJ_TREND, window, time1, price1, time2, price2);
      ObjectSetInteger(0, objName, OBJPROP_BACK, true);
      ObjectSetInteger(0, objName, OBJPROP_COLOR, clr);
      ObjectSetInteger(0, objName, OBJPROP_WIDTH, width);
      ObjectSetInteger(0, objName, OBJPROP_STYLE, style);
      ObjectSetInteger(0, objName, OBJPROP_SELECTABLE, false);
      ObjectSetInteger(0, objName, OBJPROP_SELECTED, false);
      ObjectSetInteger(0, objName, OBJPROP_HIDDEN, true);
      ObjectSetInteger(0, objName, OBJPROP_RAY, ray);
   }
   else
   {
      ObjectSetDouble(0, objName, OBJPROP_PRICE1, price1);
      ObjectSetDouble(0, objName, OBJPROP_PRICE2, price2);

      ObjectSetInteger(0, objName, OBJPROP_TIME1, time1);
      ObjectSetInteger(0, objName, OBJPROP_TIME2, time2);
   }
   
//---
}
//+------------------------------------------------------------------+
//| Create a rectangle.                                              |
//+------------------------------------------------------------------+
void CreateRect(string objName, datetime time1, double price1, datetime time2, double price2,
   color clr = clrRed, bool inBackground = TRUE)
{
//---
   
   if (ObjectFind(objName) < 0)
   {
      ObjectCreate(objName, OBJ_RECTANGLE, 0, time1, price1, time2, price2);
      ObjectSetInteger(0, objName, OBJPROP_BACK, inBackground);
      ObjectSetInteger(0, objName, OBJPROP_COLOR, clr);
      ObjectSetInteger(0, objName, OBJPROP_SELECTABLE, false);
      ObjectSetInteger(0, objName, OBJPROP_SELECTED, false);
      ObjectSetInteger(0, objName, OBJPROP_STYLE, STYLE_SOLID);
      ObjectSetInteger(0, objName, OBJPROP_HIDDEN, true);
   }
   else
   {
      ObjectSetDouble(0, objName, OBJPROP_PRICE1, price1);
      ObjectSetDouble(0, objName, OBJPROP_PRICE2, price2);

      ObjectSetInteger(0, objName, OBJPROP_TIME1, time1);
      ObjectSetInteger(0, objName, OBJPROP_TIME2, time2);
   }
   
//---
}
//+------------------------------------------------------------------+
//| Create an arrow.                                                 |
//+------------------------------------------------------------------+
void CreateArrow(string objName, datetime time, double price, uchar code = 225,
   ENUM_ARROW_ANCHOR anchor = ANCHOR_BOTTOM, ENUM_LINE_STYLE style = STYLE_SOLID, color clr = clrRed, int width = 1)
{
//---

   ObjectCreate(0, objName, OBJ_ARROW, 0, time, price);
   ObjectSetInteger(0, objName, OBJPROP_ARROWCODE, code);
   ObjectSetInteger(0, objName, OBJPROP_ANCHOR, anchor);
   ObjectSetInteger(0, objName, OBJPROP_COLOR, clr);
   ObjectSetInteger(0, objName, OBJPROP_STYLE, style);
   ObjectSetInteger(0, objName, OBJPROP_WIDTH, width);
   ObjectSetInteger(0, objName, OBJPROP_BACK, TRUE);
   ObjectSetInteger(0, objName, OBJPROP_SELECTABLE, FALSE);
   ObjectSetInteger(0, objName, OBJPROP_SELECTED, FALSE);
   ObjectSetInteger(0, objName, OBJPROP_HIDDEN, TRUE);
   ObjectSetInteger(0, objName, OBJPROP_ZORDER, 0);
   
//---
}
//+------------------------------------------------------------------+
//| Remove objects                                                   |
//+------------------------------------------------------------------+
void RemoveChartObjects(string prefix)
{
   for (int i = ObjectsTotal(); i >= 0; i--)
   {
      string objName = ObjectName(i);
      if (StringFind(objName, prefix) >= 0)
         ObjectDelete(objName);
   }
}
//+------------------------------------------------------------------+
