//+------------------------------------------------------------------+
//|                                                    FibLevels.mqh |
//|                                                          mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property strict
#ifndef FIBLVLS_NO_OUTPUT
//+------------------------------------------------------------------+
//| User Inputs                                                      |
//+------------------------------------------------------------------+
extern string              _Sep1 = "==== Labels ====";      // Labels Settings
extern bool                _ShowPriceLabels = TRUE;         // Show price labels
extern ENUM_ANCHOR_WRAPPER _LabelsAnchor = Left;            // Labels anchor
extern string              _Sep2 = "==== Base Line ====";   // Base line settings
extern ENUM_LINE_STYLE     _BaseLineStyle = STYLE_SOLID;    // Style
extern int                 _BaseLineWidth = 1;              // Width
extern color               _BaseLineColor = clrMagenta;     // Color
extern string              _Sep3 = "==== Session ====";     // Session settings
extern ENUM_LINE_STYLE     _SessLineStyle = STYLE_DOT;      // Style
extern int                 _SessLineWidth = 1;              // Width
extern color               _SessLineColor = clrWhite;       // Color
extern color               _SessRectColor = clrLime;        // Rectangle Color
extern string              _Sep4 = "==== 22.0 Line ====";   // 22.0 line settings
extern ENUM_LINE_STYLE     _220LineStyle = STYLE_DOT;       // Style
extern int                 _220LineWidth = 1;               // Width
extern color               _220LineColor = clrYellow;       // Color
extern string              _Sep5 = "==== 72.4 Line ====";   // 72.4 line settings
extern ENUM_LINE_STYLE     _724LineStyle = STYLE_DOT;       // Style
extern int                 _724LineWidth = 1;               // Width
extern color               _724LineColor = clrYellow;       // Color
#endif
//+------------------------------------------------------------------+
//| Variables                                                        |
//+------------------------------------------------------------------+
struct FIBLEVEL {
   double base;
   double _220;
   double _724;
};
//+------------------------------------------------------------------+
//| Get Fibonacci levels of instrument's base line.                  |
//+------------------------------------------------------------------+
bool GetFiboLevels(string sSymbol, int numLevels, FIBLEVEL &levels[])
{
//---

   ENUM_INSTRUMENTS instrument = GetInstrument(sSymbol);
   if (instrument == Invalid)
      return (false);
   
   int _digits = #ifdef EA_TOOLS GetDigits(sSymbol) #else _Digits #endif ,
      multiplier = 1;
	if (_digits == 3 || _digits == 5 || instrument == GOLD)
		multiplier = 10;

   double price = #ifdef EA_TOOLS GetBid(sSymbol) #else Bid #endif,
      offset = 0,
      _point = #ifdef EA_TOOLS GetPoint(sSymbol) #else Point #endif,
      baseLevel = BaseLevels[instrument],
      modifier = 63.5,
      fibPips = (modifier * _point) * multiplier;
   
   ArrayResize(levels, numLevels * 2);

   if (price < baseLevel)
      do {
         offset -= fibPips;
      } while(baseLevel + offset > price);
   else
      do {
         offset += fibPips;
      } while(baseLevel + offset < price);

   for (int i = -numLevels; i < numLevels; i++)
   {
      double level = (baseLevel + offset) + (fibPips * i),
         nextLevel = (baseLevel + offset) + (fibPips * (i + 1)),
         diff = nextLevel - level,
         base = NormalizeDouble(level, _digits),
         _724 = base + diff * 0.724,
         _220 = base + diff * 0.220;
         
      levels[i + numLevels].base = base;
      levels[i + numLevels]._220 = _220;
      levels[i + numLevels]._724 = _724;
   }
   
   return (true);
   
//---
}
#ifndef FIBLVLS_NO_OUTPUT
//+------------------------------------------------------------------+
//| Draw fibo levels.                                                |
//+------------------------------------------------------------------+
void DrawFiboLevels(FIBLEVEL &levels[])
{
//---

   for (int i = 0; i < ArraySize(levels) - 1; i++)
   {
      FIBLEVEL level = levels[i],
         levelNext = levels[i + 1];
         
      string priceLevel = _ShowPriceLabels ? " (" + DoubleToStr(level.base, _Digits) + ")" : "",
         price220 = _ShowPriceLabels ? " (" + DoubleToStr(level._220, _Digits) + ")" : "",
         price724 = _ShowPriceLabels ? " (" + DoubleToStr(level._724, _Digits) + ")" : "";
         
      CreateHLine("FIBLVLS_Base_" + IntegerToString(i), "Base " + IntegerToString(i + 1) + priceLevel, level.base, _LabelsAnchor, _BaseLineStyle, _BaseLineColor, _BaseLineWidth);
      CreateHLine("FIBLVLS_220_" + IntegerToString(i), "22.0" + price220, level._220, _LabelsAnchor, _220LineStyle, _220LineColor, _220LineWidth);
      CreateHLine("FIBLVLS_724_" + IntegerToString(i), "72.4" + price724, level._724, _LabelsAnchor, _724LineStyle, _724LineColor, _724LineWidth);
   }

   int sz = ArraySize(levels);
   FIBLEVEL level = levels[sz - 1];
   string priceLevel = _ShowPriceLabels ? " (" + DoubleToStr(level.base, _Digits) + ")" : "";
   CreateHLine("FIBLVLS_Base_" + IntegerToString(sz), "Base " + IntegerToString(sz + 1) + priceLevel, level.base, _LabelsAnchor, _BaseLineStyle, _BaseLineColor, _BaseLineWidth);
   
//---
}
//+------------------------------------------------------------------+
//| Draw session zones.                                              |
//+------------------------------------------------------------------+
void DrawSessionInformation(FIBLEVEL &levels[])
{
//---

   datetime sessStart = iTime(_Symbol, PERIOD_D1, 0),
      sessEnd = sessStart + PeriodSeconds(PERIOD_D1);
      
   CreateVLine("FIBLVLS_SessionStart", sessStart, _SessLineStyle, _SessLineColor, _SessLineWidth);
   CreateVLine("FIBLVLS_SessionEnd", sessEnd, _SessLineStyle, _SessLineColor, _SessLineWidth);
   
   double zoneAbove[2] = { EMPTY_VALUE, EMPTY_VALUE },
      zoneCenter[2] = { EMPTY_VALUE, EMPTY_VALUE },
      zoneBelow[2] = { -EMPTY_VALUE, -EMPTY_VALUE },
      price = Bid;
   bool hasZoneAbove = FALSE,
      hasZoneCenter = FALSE,
      hasZoneBelow = FALSE;

   for (int i = 1; i < ArraySize(levels) - 2; i++)
   {
      FIBLEVEL 
         levelAboveNext = levels[i + 2],
         levelNext = levels[i + 1],
         level = levels[i],
         levelPrev = levels[i - 1];
      
      
      if (levelNext._724 > price && levelNext._724 < zoneAbove[1])
      {
         hasZoneAbove = TRUE;
         zoneAbove[0] = levelAboveNext._220;
         zoneAbove[1] = levelNext._724;
      }

      if (level._220 < price && level._220 > zoneBelow[0])
      {
         hasZoneBelow = TRUE;
         zoneBelow[0] = level._220;
         zoneBelow[1] = levelPrev._724;
      }

      if (levelNext._220 > price && level._724 < price)
      {
         hasZoneCenter = TRUE;
         zoneCenter[0] = levelNext._220;
         zoneCenter[1] = level._724;
      }
      
      if (hasZoneAbove && hasZoneBelow && hasZoneCenter)
         break;
   }
   
   RemoveChartObjects("FIBLVLS_Zone");
   
   if (hasZoneAbove)
      CreateRect("FIBLVLS_ZoneAbove", sessStart, zoneAbove[0], sessEnd, zoneAbove[1], _SessRectColor); 

   if (hasZoneCenter)
      CreateRect("FIBLVLS_ZoneCenter", sessStart, zoneCenter[0], sessEnd, zoneCenter[1], _SessRectColor); 

   if (hasZoneBelow)
      CreateRect("FIBLVLS_ZoneBelow", sessStart, zoneBelow[0], sessEnd, zoneBelow[1], _SessRectColor); 
   
//---
}
#endif
//+------------------------------------------------------------------+
