//+------------------------------------------------------------------+
//|                                                    FxCalcNo1.mqh |
//|                                                          mwfx108 |
//|                                                mwfx108@gmail.com |
//+------------------------------------------------------------------+
#property copyright "mwfx108"
#property link      "mwfx108@gmail.com"
#property strict
//+------------------------------------------------------------------+
//| Enums                                                            |
//+------------------------------------------------------------------+
enum ENUM_DISPLAY_MODE {
   All_Levels        = 0,  // All
   Bias_Levels       = 1,  // Bias
   Relevant_Levels   = 2,  // Relevant
   Range_Levels      = 3   // Price Range
};
//+------------------------------------------------------------------+
//| Global Variables                                                 |
//+------------------------------------------------------------------+
int ___lastIndex = -1;
//+------------------------------------------------------------------+
//| Calculate baselines                                              |
//+------------------------------------------------------------------+
void CalculateBaselines(ENUM_INSTRUMENTS instrument, double nyClose, double &baselines[])
{
//---

   double multiplier = Multipliers[instrument],
      modifier = BaseLevels[0] * multiplier,
      bl = BaseLevels[instrument],
      diff = MathAbs(nyClose - bl);
   int sz = 2, count = (int)ceil(diff / modifier);
      
   ArrayFree(baselines);
   ArrayResize(baselines, sz);
   
   if (nyClose > bl)
   {
      baselines[0] = bl + (modifier * count);
      baselines[1] = baselines[0] - modifier;
   }
   else
   {
      baselines[0] = bl - (modifier * count);
      baselines[1] = baselines[0] + modifier;
   }
   
//---
}
//+------------------------------------------------------------------+
//| Get the Base Lines average value.                                |
//+------------------------------------------------------------------+
double GetMeanValue(double &baselines[])
{
//---

   double result = (baselines[0] + baselines[1]) / 2;
   return (result);
   
//---
}
//+------------------------------------------------------------------+
//| Sort levels                                                      |
//+------------------------------------------------------------------+
void SortLevels(BaseLevel &levels[], int direction = MODE_DESCEND)
{
//---

   BaseLevel temp;
   int n, count=0;
   bool swapped = true;
   
   n = ArraySize(levels);
   
   if (direction == MODE_ASCEND)
   {
      while (swapped)
      {
         swapped = false;
         count++;
         for (int i = 1; i < n; i++)
         {
            if (levels[i - 1].price > levels[i].price)
            {
               temp = levels[i];
               levels[i] = levels[i - 1];
               levels[i - 1] = temp;
               swapped = true;
            }
         }
      }
   }
   else if (direction == MODE_DESCEND)
   {
      while (swapped)
      {
         swapped = false;
         count++;
         for (int i = 0; i < n - 1; i++)
         {
            if(levels[i].price < levels[i + 1].price)
            {
               temp = levels[i + 1];
               levels[i + 1] = levels[i];
               levels[i] = temp;
               swapped = true;
            }
         }
      }
   }
   
//---
}
//+------------------------------------------------------------------+
//| Calculate levels                                                 |
//+------------------------------------------------------------------+
void GetLevels(string symbol, double nyClose, ENUM_ORDER_TYPE &bias, BaseLevel &levels[])
{
//---

   ArrayFree(levels);
   ArrayResize(levels, ArraySize(LEVEL_LABELS));
   
   ___lastIndex = -1;

   ENUM_INSTRUMENTS instrument = GetInstrument(symbol);
   if (instrument == Invalid)
      return;

   // ---- Calculate baselines and mean value
   double multiplier = Multipliers[instrument];
   double baselines[], mv;
   
   double nyc = NormalizeDouble(nyClose, _Digits - 1);
   
   ArrayResize(baselines, 2);
   CalculateBaselines(instrument, nyClose, baselines);
   mv = GetMeanValue(baselines);
   ArrayFree(baselines);
   // ----
   
   // ---- Prepare
   double levelsBuy[], levelsSell[],
      tpStepsBuy[], tpStepsSell[],
      hb[], lb[], 
      entriesSell[], sellTPs[], 
      entriesBuy[], buyTPs[];
   
   ArrayResize(levelsBuy, ArraySize(Levels_Buy));
   for (int i = 0; i < ArraySize(levelsBuy); i++)
      levelsBuy[i] = Levels_Buy[i] * multiplier;
   ArrayResize(levelsSell, ArraySize(Levels_Sell));
   for (int i = 0; i < ArraySize(levelsSell); i++)
      levelsSell[i] = Levels_Sell[i] * multiplier;
      
   ArrayResize(tpStepsBuy, ArraySize(TP_Steps_Buy));
   for (int i = 0; i < ArraySize(tpStepsBuy); i++)
      tpStepsBuy[i] = TP_Steps_Buy[i] * multiplier;
   ArrayResize(tpStepsSell, ArraySize(TP_Steps_Sell));
   for (int i = 0; i < ArraySize(tpStepsSell); i++)
      tpStepsSell[i] = TP_Steps_Sell[i] * multiplier;
      
   ArrayResize(hb, 3);
   ArrayResize(lb, 3);
   ArrayResize(entriesSell, 6);
   ArrayResize(entriesBuy, 6);
   ArrayResize(sellTPs, 4);
   ArrayResize(buyTPs, 4);
   // ----

   int levelIndex = 0,
      digits = (int)MarketInfo(symbol, MODE_DIGITS);

   // --- NYC
   levels[levelIndex].price = nyc;
   levels[levelIndex].type = LEVEL_NYC;
   levels[levelIndex].index = levelIndex;
   levels[levelIndex].subIndex = -1;
#ifndef FXCALC_NO_OUTPUT
   levels[levelIndex].lineColor = Color_NYC;
   levels[levelIndex].lineStyle = Style_NYC;
   levels[levelIndex].lineWidth = Width_NYC;
#endif   
   levelIndex++;
      
   // ---- Pivot and sentiment
   double pivot = 0.0;
   double sentiment = 0.0;
   if (nyc < mv)
   {
      pivot = nyc * PV[0] + mv * PV[1] - (0.00105 * multiplier);
      sentiment = nyc * MS[0] + mv * MS[1] + (0.0015 * multiplier);;
   }
   else
   {
      pivot = nyc * PV[0] + mv * PV[1] + (0.001 * multiplier);
      sentiment = nyc * MS[0] + mv * MS[1] + (0.00155 * multiplier);;
   }
   
   levels[levelIndex].price = NormalizeDouble(pivot, digits);
   levels[levelIndex].type = LEVEL_PIVOT;
   levels[levelIndex].index = levelIndex;
   levels[levelIndex].subIndex = -1;
#ifndef FXCALC_NO_OUTPUT
   levels[levelIndex].lineColor = Color_Pivot;
   levels[levelIndex].lineStyle = Style_Pivot;
   levels[levelIndex].lineWidth = Width_Pivot;
#endif   
   levelIndex++;
   
   levels[levelIndex].price = NormalizeDouble(sentiment, digits);
   levels[levelIndex].type = LEVEL_SENTIMENT;
   levels[levelIndex].index = levelIndex;
   levels[levelIndex].subIndex = -1;
#ifndef FXCALC_NO_OUTPUT
   levels[levelIndex].lineColor = Color_Sentiment;
   levels[levelIndex].lineStyle = Style_Sentiment;
   levels[levelIndex].lineWidth = Width_Sentiment;
#endif   
   levelIndex++;
   // ----

   // ---- Sell entries
   for (int i = 0; i < ArraySize(levelsSell); i++)
      entriesSell[i] = nyc + levelsSell[i];
      
   for (int i = 0; i < ArraySize(entriesSell); i++)
   {
      levels[levelIndex].price = NormalizeDouble(entriesSell[i], digits);
      levels[levelIndex].type = LEVEL_SELL;
      levels[levelIndex].index = levelIndex;
      levels[levelIndex].subIndex = i;
#ifndef FXCALC_NO_OUTPUT
      levels[levelIndex].lineColor = Color_Sell_Entries;
      levels[levelIndex].lineStyle = Style_Sell_Entries;
      levels[levelIndex].lineWidth = Width_Sell_Entries;
#endif   
      levelIndex++;
   }
   // ----

   // ---- Sell Take Profit points
   for (int i = 0; i < ArraySize(tpStepsSell); i++) 
      sellTPs[i] = nyc - tpStepsSell[i]; 
      
   for (int i = 0; i < ArraySize(sellTPs); i++)
   {
      levels[levelIndex].price = NormalizeDouble(sellTPs[i], digits);
      levels[levelIndex].type = LEVEL_STP;
      levels[levelIndex].index = levelIndex;
      levels[levelIndex].subIndex = i;
#ifndef FXCALC_NO_OUTPUT
      levels[levelIndex].lineColor = Color_Sell_Targets;
      levels[levelIndex].lineStyle = Style_Sell_Targets;
      levels[levelIndex].lineWidth = Width_Sell_Targets;
#endif   
      levelIndex++;
   }
   // ----

   // ---- Buy entries
   for (int i = 0; i < ArraySize(levelsBuy); i++)
      entriesBuy[i] = nyc - levelsBuy[i];
      
   for (int i = 0; i < ArraySize(entriesBuy); i++)
   {
      levels[levelIndex].price = NormalizeDouble(entriesBuy[i], digits);
      levels[levelIndex].type = LEVEL_BUY;
      levels[levelIndex].index = levelIndex;
      levels[levelIndex].subIndex = i;
#ifndef FXCALC_NO_OUTPUT
      levels[levelIndex].lineColor = Color_Buy_Entries;
      levels[levelIndex].lineStyle = Style_Buy_Entries;
      levels[levelIndex].lineWidth = Width_Buy_Entries;
#endif   
      levelIndex++;
   }
   // ----

   // ---- Buy Take Profit points
   for (int i = 0; i < ArraySize(tpStepsBuy); i++) 
      buyTPs[i] = nyc + tpStepsBuy[i]; 
      
   for (int i = 0; i < ArraySize(buyTPs); i++)
   {
      levels[levelIndex].price = NormalizeDouble(buyTPs[i], digits);
      levels[levelIndex].type = LEVEL_BTP;
      levels[levelIndex].index = levelIndex;
      levels[levelIndex].subIndex = i;
#ifndef FXCALC_NO_OUTPUT
      levels[levelIndex].lineColor = Color_Buy_Targets;
      levels[levelIndex].lineStyle = Style_Buy_Targets;
      levels[levelIndex].lineWidth = Width_Buy_Targets;
#endif   
      levelIndex++;
   }
   // ----
   
   // ---- High break outs
   if (nyc < mv)
      hb[0] = nyc * HB1[0] + mv * HB1[1] - (0.00115 * multiplier);
   else
      hb[0] = nyc * HB1[0] + mv * HB1[1] + (0.0042 * multiplier);
   hb[1] = pivot + (0.0039 * multiplier);
   hb[2] = nyc * PV[0] + mv * PV[1] + (0.0095 * multiplier);
   
   for (int i = 0; i < ArraySize(hb); i++)
   {
      levels[levelIndex].price = NormalizeDouble(hb[i], digits);
      levels[levelIndex].type = LEVEL_HBO;
      levels[levelIndex].index = levelIndex;
      levels[levelIndex].subIndex = i;
#ifndef FXCALC_NO_OUTPUT
      levels[levelIndex].lineColor = Color_HBO_Levels;
      levels[levelIndex].lineStyle = Style_HBO_Levels;
      levels[levelIndex].lineWidth = Width_HBO_Levels;
#endif   
      levelIndex++;
   }
   // ----

   // ---- Low break outs
   if (nyc < mv)
      lb[0] = hb[0] - (0.00315 * multiplier);
   else
      lb[0] = nyc * LB1[0] + mv * LB1[1];
   lb[1] = pivot - (0.0039 * multiplier);
   if (nyc < mv)
      lb[2] = nyc * HB1[0] + mv * HB1[1] - (0.0085 * multiplier);
   else
      lb[2] = nyc * HB1[0] + mv * HB1[1] - (0.0095 * multiplier);
      
   for (int i = 0; i < ArraySize(lb); i++)
   {
      levels[levelIndex].price = NormalizeDouble(lb[i], digits);
      levels[levelIndex].type = LEVEL_LBO;
      levels[levelIndex].index = levelIndex;
      levels[levelIndex].subIndex = i;
#ifndef FXCALC_NO_OUTPUT
      levels[levelIndex].lineColor = Color_LBO_Levels;
      levels[levelIndex].lineStyle = Style_LBO_Levels;
      levels[levelIndex].lineWidth = Width_LBO_Levels;
#endif   
      levelIndex++;
   }
   // ----
   
   // ---- New Monthly High   
   double nmh = nyc * HB1[0] + mv * HB1[1] + (0.0127 * multiplier);
   
   levels[levelIndex].price = NormalizeDouble(nmh, digits);
   levels[levelIndex].type = LEVEL_NMH;
   levels[levelIndex].index = levelIndex;
   levels[levelIndex].subIndex = -1;
#ifndef FXCALC_NO_OUTPUT
   levels[levelIndex].lineColor = Color_NMH;
   levels[levelIndex].lineStyle = Style_NMH;
   levels[levelIndex].lineWidth = Width_NMH;
#endif   
   levelIndex++;
   // ----
   
   SortLevels(levels);
   
   // ---- Bias
   if (pivot < sentiment)
      bias = OP_BUY;
   else
      bias = OP_SELL;
   // ----
   
   ArrayFree(hb);
   ArrayFree(lb);
   ArrayFree(entriesSell);
   ArrayFree(entriesBuy);
   ArrayFree(sellTPs);
   ArrayFree(buyTPs);
   
//---
}
//+------------------------------------------------------------------+
//| Get levels near current price                                    |
//+------------------------------------------------------------------+
void GetClosestLevelIndizes(BaseLevel &levels[], double price, bool considerBias, ENUM_ORDER_TYPE bias,
   int &indexAbove, int &indexBelow)
{
//---

   double levelAbove = EMPTY_VALUE,
      levelBelow = -EMPTY_VALUE;
      
   indexAbove = -1;
   indexBelow = -1;
      
   for (int i = 0; i < ArraySize(levels); i++)
   {
      bool relevantLevel = TRUE;
      if (considerBias)
         relevantLevel = IsRelevantLevel(bias, levels[i]);
   
      if (levels[i].price < levelAbove && levels[i].price >= price && relevantLevel)
      {
         levelAbove = levels[i].price;
         indexAbove = i; //levels[i].index;
      }
      else if (levels[i].price > levelBelow && levels[i].price <= price && relevantLevel)
      {
         levelBelow = levels[i].price;
         indexBelow = i; //levels[i].index;
      }
   }
   
//---
}
//+------------------------------------------------------------------+
//| Get level closest to current price                               |
//+------------------------------------------------------------------+
int GetClosestLevelIndex(BaseLevel &levels[], double price, bool considerBias, ENUM_ORDER_TYPE bias)
{
//---

   int result = -1;

   double levelDiff = EMPTY_VALUE;
      
   for (int i = 0; i < ArraySize(levels); i++)
   {
      if (considerBias && !IsRelevantLevel(bias, levels[i]))
         continue;
         
      double diff = MathAbs(levels[i].price - price);
      if (diff < levelDiff)
      {
         result = i;
         levelDiff = diff;
      }
   }
   
   return (result);
   
//---
}
//+------------------------------------------------------------------+
//| Determine if level is relevant                                   |
//+------------------------------------------------------------------+
bool IsRelevantLevel(ENUM_ORDER_TYPE bias, BaseLevel &level)
{
//---
   
   bool result = FALSE;

   switch (bias)
   {
      case OP_BUY:
         result =
            level.type == LEVEL_BUY ||
            level.type == LEVEL_BTP ||
            level.type == LEVEL_HBO;
         break;
      case OP_SELL:
         result =
            level.type == LEVEL_SELL ||
            level.type == LEVEL_STP ||
            level.type == LEVEL_LBO;
         break;
   }
   
   result = result || 
      (level.type == LEVEL_NYC ||
       level.type == LEVEL_PIVOT ||
       level.type == LEVEL_SENTIMENT ||
       level.type == LEVEL_NMH);
       
   return (result);  
//---
}
#ifndef FXCALC_NO_OUTPUT
//+------------------------------------------------------------------+
//| Draw levels                                                      |
//+------------------------------------------------------------------+
void DrawLevels(double nyClose, double price, BaseLevel &levels[], ENUM_ORDER_TYPE bias,
   ENUM_DISPLAY_MODE displayMode, bool showBreakLines)
{
//---

   if (ArraySize(levels) == 0)
      return;

   for (int i = 0; i < ArraySize(levels); i++)
   {
      string name = LEVEL_LABELS[levels[i].index];
      bool showLevel = TRUE;
      
      if (displayMode == Relevant_Levels)
      {
         int indexAbove = -1,
            indexBelow = -1;
            
         GetClosestLevelIndizes(levels, price, FALSE, bias, indexAbove, indexBelow);
         
         showLevel = (indexAbove == i) || (indexBelow == i);
      }
      else if (displayMode == Range_Levels)
      {
         showLevel =
            (levels[i].price + 140 * Point >= price) && 
            (levels[i].price - 140 * Point <= price);
      }
      else if (displayMode == Bias_Levels)
      {
         showLevel = IsRelevantLevel(bias, levels[i]);
      }
      
      if (showLevel)
         CreateHLine("FXC_" + name, name, levels[i].price,
            Right, levels[i].lineStyle, levels[i].lineColor, levels[i].lineWidth);
      else
         RemoveChartObjects("FXC_" + name);
   }

   if (showBreakLines)
   {
      if (___lastIndex < 0)
      {
         ___lastIndex = GetClosestLevelIndex(levels, price, displayMode == Bias_Levels, bias);
         if (___lastIndex < 0)
         {
            return;
         }
      }
      else
      {
         if (price > levels[___lastIndex].price + 140 * Point || price < levels[___lastIndex].price - 140 * Point)
         {
            ___lastIndex = -1;
            return;
         }
      }

      RemoveChartObjects("FXC_Break");
      string name = LEVEL_LABELS[levels[___lastIndex].index];
      CreateHLine("FXC_Break_" + name + "_Above", name + " Break High", levels[___lastIndex].price + 140 * Point,
         Right, Style_Break_Levels, Color_Break_Levels, Width_Break_Levels);
      CreateHLine("FXC_Break_" + name + "_Below", name + " Break Low", levels[___lastIndex].price - 140 * Point,
         Right, Style_Break_Levels, Color_Break_Levels, Width_Break_Levels);
   }
   
//---
}
//+------------------------------------------------------------------+
//| Output bias information                                          |
//+------------------------------------------------------------------+
void DrawBiasInformation(bool up)
{
//---
   
   string objLabelName = "FXC_Bias_Label",
      objLabelCaption = "Bias: ";
      
   color objLabelColor;
      
   if (up)
   {
      objLabelColor = Color_Bias_Buy;
      objLabelCaption = objLabelCaption + "BUY";
   }
   else
   {
      objLabelColor = Color_Bias_Sell;
      objLabelCaption = objLabelCaption + "SELL";
   }
   
   int x, y, w, h;
   TextGetSize(objLabelCaption, w, h);
   x = 0;
   y = h;
   
   if (ObjectFind(objLabelName) < 0)
   {   
      ObjectCreate(objLabelName, OBJ_LABEL, 0, 0, 0);
      ObjectSetInteger(0, objLabelName, OBJPROP_BACK, true);
      ObjectSetInteger(0, objLabelName, OBJPROP_COLOR, objLabelColor);
      ObjectSetInteger(0, objLabelName, OBJPROP_XDISTANCE, x + 4);
      ObjectSetInteger(0, objLabelName, OBJPROP_YDISTANCE, y - h / 2);
      ObjectSetInteger(0, objLabelName, OBJPROP_XSIZE, w);
      ObjectSetInteger(0, objLabelName, OBJPROP_YSIZE, h);
      ObjectSetInteger(0, objLabelName, OBJPROP_ANCHOR, ANCHOR_RIGHT);
      ObjectSetInteger(0, objLabelName, OBJPROP_CORNER, CORNER_RIGHT_UPPER);
      ObjectSetInteger(0, objLabelName, OBJPROP_BACK, false);
      ObjectSetInteger(0, objLabelName, OBJPROP_SELECTABLE, false);
      ObjectSetInteger(0, objLabelName, OBJPROP_SELECTED, false);
      ObjectSetInteger(0, objLabelName, OBJPROP_HIDDEN, true);
   }
   
   ObjectSetText(objLabelName, objLabelCaption, 11);
   
//---
}
//+------------------------------------------------------------------+
//| Remove objects                                                   |
//+------------------------------------------------------------------+
void RemoveObjects(bool removeBias = TRUE)
{
   for (int i = ObjectsTotal(); i >= 0; i--)
   {
      string objName = ObjectName(i);
      if (StringFind(objName, "FXC_") >= 0 && removeBias)
         ObjectDelete(objName);
      else if (StringFind(objName, "FXC_") >= 0 && !removeBias && StringFind(objName, "FXC_Bias_") < 0)
         ObjectDelete(objName);
   }
}
#endif
//+------------------------------------------------------------------+
